/**
 * Copyright 2005 Simon Lehmayr
 *
 * This file is part of Asteroid Zone. Asteroid Zone is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version.
 * Asteroid Zone is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Asteroid Zone; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

package asteroids;

import javax.microedition.lcdui.*;
import java.util.*;

/**
 * Class to implement the ufo. The ufo
 * can move and shoot a homing missile.
 * @author Simon Lehmayr
 * @version 1.6, 2005/12/20
 */
public class Ufo extends Mobile {
    /** False if the ufo has been shot. */
    public boolean isAlive = true, isActive = false;

    private static final int FIRESPEED[] = {1000, 700, 300};
    private static final int VALUEUFO[] = {10, 20, 35};

    /** Counters */
    private int holdOffMovement, holdOffShoot, holdOffUfoSound;
    
    /** The ufo singleton */
    public static Ufo ufo;

    /** Coordinates */
    public static final byte[] pointx = {-6,-3, 3, 6, 4, 1, 1, -1, -1, -4};
    public static final byte[] pointy = {0, 3, 3, 0, -3, -3, -5, -5, -3, -3};
    public static final byte _radiusx = 5;
    public static final byte _radiusy = 3;

    private byte _explosionFrame;
    public int Direction;

    static {
        // Instantiate the ship.
        ufo = new Ufo();
    }

    public Ufo() {
    }

    /**
     * Positions the ship at its original location, orientation
     * and velocity.
     */
    public final void reset() {
        Field field = Game.field;
        Random rnd = new Random();
        Direction = rnd.nextInt(2)*2 - 1;
        if (Direction > 0) moveTo(Mobile.width - 1, rnd.nextInt(Mobile.height-10) +5); else moveTo(1, rnd.nextInt(Mobile.height-10) +5);
        vx = (-250 - 50 * field._level) * Direction;
        vy = (rnd.nextInt(401) - 200);
        holdOffMovement = rnd.nextInt(41) + 15;
        holdOffShoot = (rnd.nextInt(201) + 100) / (field.difficulty + 1);
        isAlive = true;
        isActive = false;
        holdOffUfoSound = 0;
//	    holdOffUfo = (rnd.nextInt(501 - field._level * 50) + 300 - field._level * 30) / (field.difficulty + 1);
//        System.out.println("Ufo :: reset");
    }

    /**
     * Called when the ship collides with an asteroid.
     */
    public final void explode() {
        isAlive = false;
        _explosionFrame = 0;
    }

    /**
     * Called to start the ufo action.
     */
    public final void startUfo() {
        isActive = true;
    }

    /**
     * Move the ship to its next position. Returns true when ufo wants to shoot.
     */
    public final void move() {
        Random rnd = new Random();
        Field field = Game.field;
        Soundsystem sound = Game.sound;
//        System.out.println("Ufo :: move - holdOffUfo="+holdOffUfo);
//        if (holdOffUfo == 0) isActive = true; else holdOffUfo--;
        if (isActive) 
        {
            if (isAlive) {
                if (holdOffUfoSound == 0) {sound.playSound(sound.UFO); holdOffUfoSound = 55;} else holdOffUfoSound--;
                xold = x;
                yold = y;
                _x += vx;
                _y += vy;
                x = _x >> 8;
                y = _y >> 8;
    //            if (x <= 2) {reset(); System.out.println("Ufo :: links erreicht"); return false;}
    //            System.out.println("Ufo :: move - holdOffMovement="+holdOffMovement+", vx="+vx+", vy="+vy);
                holdOffMovement--;
                holdOffShoot--;
                if (holdOffShoot == 0) {holdOffShoot = (rnd.nextInt(201) + 100) / (field.difficulty + 1); Missile.missile.fire();}
                if (holdOffMovement == 0) {vy = (rnd.nextInt(201) - 100); holdOffMovement = rnd.nextInt(41) + 15;}
                if (y < 10) {vy = rnd.nextInt(201); holdOffMovement = rnd.nextInt(41) + 15;}
                else if (y > height - 10) {vy = -rnd.nextInt(201); holdOffMovement = rnd.nextInt(41) + 15;}
                
                // If a border has been hit, wrap the trajectory around
                // the screen. The new origin is the projection of the
                // intersection point on the opposite border.
                if (x <= 0) {
                    moveTo(width - 2, y);
                } else if (x >= width - 1) {
                    moveTo(1, y);
                } else if (y <= 0) {
                    moveTo(x, height - 2);
                } else if (y >= height - 1) {
                    moveTo(x, 1);
                }
            } else {
                _explosionFrame++;
                if (_explosionFrame > 20) {
                    reset();
//                    System.out.println("Ufo: Explosion");
                }
            }
        }
    }

    /**
     * Compute collisions between the ufo and
     * the ship and the rockets.
     */
    public static final void collisionDetection() {
        Field field = Game.field;
        Ship ship = Ship.ship;
        Soundsystem sound = Game.sound;
        if (!ufo.isActive || !ufo.isAlive) return;
            
        if ((field != null) && (field.getState() == Field.GAME_STATE)) {
            
            int score = field.getScore();
            int newScore = score;
            Pool rockets = Rocket.rockets;
            Pool explosions = Explosion.explosions;
            
            if (!ship.isShieldActive) {
                // Detect ship - ufo collisions.
                int offset = ship.angle << 2;
                if (!ship.isInvincible && ship.isAlive) for (int i = 0; i < 4; i++) {
                    int sx = ship.x + Ship.xcoords[offset + i] - ufo.x + _radiusx;
                    if ((sx >= 0) && (sx < 2 * _radiusx)) {
                        int sy = ship.y + Ship.ycoords[offset + i] - ufo.y + _radiusy;
                        if ((sy >= 0) && (sy < _radiusy)) {
                            int lives = field.getLives();
                            if (lives > 0) {
                                sound.playSound(sound.CRASH);
                                ship.explode();
                                ufo.explode();
                                field.setLives(lives - 1);
                            }
                        }
                    }
                }
            }
            else {
                if (distance(ship.x, ship.y, ufo.x, ufo.y) <= ship.SHIELD_RADIUS + _radiusy) {
                    sound.playSound(sound.CRASH);
                    ufo.explode();
                    ship.shields -= VALUEUFO[field.difficulty]; if (ship.shields < 0) ship.shields = 0;
                }
            }

            // Detect ufo - rocket collisions.
            if (ufo.isAlive) for (rockets.current = rockets.count - 1; rockets.current >= 0;) {
                Rocket r = (Rocket)rockets.pool[rockets.current--];
                
                // Transform the r into ufo coordinates.
                int rx = r.x - ufo.x + _radiusx;
                if ((rx >= 0) && (rx < 2*_radiusx)) {
                    int ry = r.y - ufo.y + _radiusy;
                    if ((ry >= 0) && (ry < 2*_radiusy)) {
                        rockets.removeCurrent();
                        newScore += VALUEUFO[field.difficulty];
                        ufo.explode();
//                        System.out.println("Ufo :: Erschossen");
                        sound.playSound(sound.EXPLOSION);
                        break;
                    }
                }
            }

            if (newScore != score) {
                field.setScore(newScore);
            }
        }
    }
         
    /**
     * Draws the ship at the specified location.
     */
    public static void draw(int xpos, int ypos, Graphics g) {
        int i;
        for (i=0; i<pointx.length-1; i++)
            g.drawLine(pointx[i] + xpos, pointy[i] + ypos,
		pointx[i+1] + xpos, pointy[i+1] + ypos);
        g.drawLine(pointx[i] + xpos, pointy[i] + ypos,
	    pointx[0] + xpos, pointy[0] + ypos);
    }

    /**
     * Draws the ship in the specified graphic context.
     */
    public final void draw(Graphics g) {
        if (!isActive) return;
        if (isAlive) {
            draw(x, y, g);
        } else {
            if (_explosionFrame < 6) {
                g.setColor(Field.COLOR_EXPLOSION);
                int radius = _explosionFrame;
                g.drawArc(x - radius, y - radius, radius << 1, radius << 1, 0, 360);
                radius = _explosionFrame + 2;
                g.drawArc(x - radius, y - radius, radius << 1, radius << 1, 0, 360);
                radius = _explosionFrame + 4;
                g.drawArc(x - radius, y - radius, radius << 1, radius << 1, 0, 360);
            }
        }
    }
}

